/* RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and that
no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
					scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
					markk@internic.net

File: procrc.c
Developed by: Scott Williamson

*/
#include "rwhois.h"

procrc(cfile)
char *cfile;
{
FILE *fp;
char temp[MAXLINE];
char cmd[10], act[80], act1[80];
int servnum = 0;
int limit;
char line[MAXLINE];
int param_err  = FALSE;
char secondfile[MAXBUFF];
char *ptr;


	if( cfile)
	{
		if(verbose > TALK)
		{
			sprintf(temp, "Trying to open [%s] config file... ", cfile);
			note( temp);
		}

		if( !(fp = fopen( cfile, "r")))
		{
			strcpy( secondfile, cfile);

			if( ptr =	rindex( cfile, '.'))
			{
				if(strcmp( ptr, ".rc"))
				{
					strcat( secondfile, ".rc");
				}

			}

			if( !(fp = fopen( secondfile, "r")))
			{
				if( verbose > TALK )
				{
			sprintf(temp,"Could not open [%s] or [%s] configuration file...", 
					cfile, secondfile);
					warning( temp);
				}
				return(-1);
			}
		}
		else
		{
			if( verbose > TALK )
			{
				sprintf(temp, "[%s] configuration file open...", cfile);
				note( temp);
			} 
		} 
	}
	else
	{
		error("Need config file name");
		return( -1);
	}

	while(fgets(  line, MAXLINE, fp))
	{
	
		sscanf( line,  "%s %s %s", cmd, act, act1);
		upper( cmd);

		if ( *cmd == '#' )
			; /* comment do nothing */

		else if((!strncmp( cmd, "SERVER", 6)) && servnum < MAXSERVERS)
		{
			upper( act);
			if(!strncmp( act, "ADD", 3))
				cmdline_server( act1, REF_CONFFILE);
			else if(!strncmp( act, "CHANGE", 6))
			{
				SERVERREWIND;
				referral = 0;
				cmdline_server( act1, REF_CONFFILE);
			}
		}
		else if(!strncmp( cmd, "USER", 4))
		{
			user = safecalloc( 1, strlen(act)+1);
			strcpy( user, act);
			if( verbose > WHISPER)
			{
				sprintf(temp, "Seting the user id to [%s]\n", user);
				note( temp);
			}
		}
		else if(!strncmp( cmd, "AUTH", 4))
		{
			authstring = safecalloc( 1, strlen(act)+1);
			strcpy( authstring, act);
			if( verbose > WHISPER)
				note("Set the authority string");
		}
		else if(!strncmp( cmd, "SHOTS", 5))
		{
			upper( act);
			if(!strncmp( act, "MULTI", 5))
			{
				if( verbose > WHISPER )
					note("Switching to multi referral mode");
				shots = MULTISHOT;
			}
			else if(!strncmp( act, "SINGLE", 5))
			{
				if( verbose > WHISPER )
					note("Switching to single referral mode");
				shots = SINGLESHOT;
			}
			else
			{
				param_err = TRUE;
				shots = DEFAULTSHOT;
			}
		}
		else if(!strncmp( cmd, "MIME", 4))
		{
			mimepath = safecalloc( 1,  (strlen( act)+1));
			strcpy( mimepath, act);
			if( verbose > WHISPER)
			{
				sprintf( temp, "Seting the mime reader path to [%s]"
					, mimepath);
				note( temp);
			}
			display_cap = display_cap|DISPLAY_MIME;
			/*  add this in when server ready   */
			control_pannel[CP_DISPLAY] = SET_SEND|SET_ALWAYS;
		}
		else if(!strncmp( cmd, "URL", 3))
		{
			urlpath = safecalloc( 1,  (strlen( act)+1));
			strcpy( urlpath, act);
			if( verbose > WHISPER)
			{
				sprintf( temp, "Seting the url follower to [%s]", urlpath);
				note( temp);
			}
			display_cap = display_cap|DISPLAY_URL;
			/* add this in when server ready  */
			control_pannel[CP_DISPLAY] = SET_SEND|SET_ALWAYS;
		}
		else if(!strncmp( cmd, "EDITOR", 6))
		{
			editpath = safecalloc( 1,  (strlen( act)+1));
			strcpy( editpath, act);
			if( verbose > WHISPER)
			{
				sprintf( temp, "Seting editior to [%s]", editpath);
				note( temp);
			}
		}
		else if(!strncmp( cmd, "VERBOSE", 6))
		{
			upper( act);
			if(!strncmp( act, "TALK", 2))
				verbose = TALK;
			else if(!strncmp( act, "SHOUT", 5))
				verbose = SHOUT;
			else if(!strncmp( act, "QUIET", 5))
				verbose = QUIET; 
			else if(!strncmp( act, "WHISPER", 7))
				verbose = WHISPER;
			else
				param_err = TRUE;
		}
		else if(!strncmp( cmd, "FORWARD", 7))
		{
			if(!strncmp( act, "ON", 2))
			{
				default_forward = FORWARD_ON;
				if( verbose > TALK)
					note("Request FORWARD set to ON");
			}
			else if(!strncmp( act, "OFF", 3))
			{
				default_forward = FORWARD_OFF;
				if( verbose > TALK)
					note("Request FORWARD set to OFF");
			}
			else
				param_err = TRUE;

		}
		else if(!strncmp( cmd, "CACHE", 5))
		{
			upper( act);
			if(!strncmp( act1, "ON", 2))
			{
				default_cache = ON;
				if( verbose > TALK)
					note("Request CACHE set to ON");
			}
			else if(!strncmp( act1, "OFF", 3))
			{
				default_cache = OFF;
				if( verbose > TALK)
					note("Request CACHE set to OFF");
			}
			else
				param_err = TRUE;
		}
		else if(!strncmp( cmd, "LOCALCACHE", 5))
		{
			upper( act);
			if(!strncmp( act1, "ON", 2))
			{
				default_lcache = ON;
				if( verbose > TALK)
					note("Local CACHE set to ON");
			}
			else if(!strncmp( act1, "OFF", 3))
			{
				default_lcache = OFF;
				if( verbose > TALK)
					note("Local CACHE set to OFF");
			}
			else
				param_err = TRUE;
		}
		else if(!strncmp( cmd, "CONFDIR", 7))
		{
			confdir = safecalloc( 1,  (strlen( act)+1));
			strcpy( confdir, act);
			if( verbose > WHISPER)
			{
				sprintf( temp, "Seting the config directory to [%s]", confdir);
				note( temp);
			}
		}
		else if(!strncmp( cmd, "CACHEDIR", 8))
		{
			cachedir = safecalloc( 1,  (strlen( act)+1));
			strcpy( cachedir, act);
			if( verbose > WHISPER)
			{
				sprintf( temp, "Seting local cache directory to [%s]", 
					cachedir);
				note( temp);
			}
		}
		else if(!strncmp( cmd, "SET", 6))
		{
			upper( act);
			if(!strncmp( act, "WAIT", 4))
			{
				default_wait = atoi( act1);
				if( verbose > TALK)
				{
					sprintf( temp, "Setting WAIT to %d", default_wait);
					note(temp);
				}
			}
			else if(!strncmp( act, "LOAD", 4))
			{
				default_load = atoi( act1);
				if( verbose > TALK)
				{
					sprintf( temp, "Setting LOAD limit to %d", default_load);
					note(temp);
				}
			}
			else if(!strncmp( act, "LIMIT", 5))
			{
				limit = atoi( act1);
				send_limit( limit, LIMIT_SET);
				control_pannel[CP_LIMIT] = SET_SEND|SET_ALWAYS;
				if( verbose > TALK)
				{
					sprintf( temp, "Will request a LIMIT of %d", limit);
					note(temp);
				}

			}
			else if(!strncmp( act, "PORT", 4))
			{
				default_port = atoi( act1);
				if( verbose > TALK)
				{
					sprintf( temp, "Setting PORT to %d", default_port);
					note(temp);
				}
			}
			else
				param_err = TRUE;
		}
		else
		{
			sprintf( temp, "Invalid config file instruction [%s]", cmd);
			error( temp);
		}

		if( param_err == TRUE)
		{
			sprintf( temp, "Invalid argument [%s] for the [%s] command",
				act, cmd);
			error( temp);
			param_err = FALSE;
		}
			
	}

	fclose(fp);
	return(0);

}

