/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "misc.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <sys/types.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#include "client_msgs.h"
#include "defines.h"
#include "log.h"
#include "strutil.h"

/* readline: reads a line from stdin (or in daemon mode, the socket,
      which is dup()d to stdin).  Strips the trailing terminating
      characters from the line. Returns a pointer to the buffer itself */
char *
readline(buffer, size)
  char *buffer;
  int  size;
{
  if (! fgets(buffer, size, stdin))
  {
    *buffer = '\0';
  }

  /* remove all nasty control characters */
  strip_control(buffer);
  
  return(buffer);
}

/* new_record: tests to see if 'line' is a record separator.  Returns
      TRUE if it is, FALSE if not */
int
new_record(line)
  char *line;
{
  if (!line || !*line)
  {
    return FALSE;
  }
  
  stripchar(line, '\n');

  if (! strncmp(line, "---", 3))
  {
    return(TRUE);
  }

  return(FALSE);
}

/* parse_line: splits a <tag>: <datum> line into its respective
      components. */
int
parse_line(line, tag, datum)
  char *line;
  char *tag;
  char *datum;
{
  char *colon = NULL;
  char *pound = NULL;
  char *d_ptr = NULL;
  
  /* first, bail out if we have an empty string */
  if (!line && !*line)
  {
    return FALSE;
  }

  pound = strchr(line, '#');
  if (pound)
  {
    *pound = '\0';
  }
  
  /* second, remove undesirable possible linefeed characters */
  stripchar(line, '\n');
  stripchar(line, '\r');

  /* now, find the separator character */
  if ( !(colon = strchr(line, ':')) )
  {
    /* it wasn't there so this is an invalid or commented line */
    return FALSE;
  }

  *colon = '\0';
  d_ptr = colon + 1;
  
  strcpy(tag, line);
  strcpy(datum, d_ptr);

  /* remove excess whitespace */
  trim(tag);
  trim(datum);

  /* put the colon back */
  *colon = ':';
  
  return TRUE;
}

/* delimstr: retrieves 'cnt' delimiters from mainbuf into
      cpbuf. Returns TRUE if successful, FALSE if not. */
int
delimstr(mainbuf, delim, cpbuf, cnt)
  char *mainbuf;
  char *delim;
  char *cpbuf;
  int  cnt;
{
  int           i           = 0;
  char          *token;
  static char   *cpbuff     = NULL;
  static int    cpbuff_size = 0;

  if (strlen(mainbuf) + 1 > cpbuff_size)
  {
    if (cpbuff == NULL)
    {
      cpbuff =  (char *)malloc ((strlen (mainbuf)+1)*sizeof(char));
    }
    else
    {
      cpbuff = (char *)realloc (cpbuff, strlen(mainbuf) + 1);
    }
    if (cpbuff == NULL)
    {
      log_error("delimstr: error requesting memory");
      return FALSE;
    }

    cpbuff_size = strlen(mainbuf) + 1;
  }

  strcpy(cpbuff, mainbuf);

  token = strtok(cpbuff, delim);

  while ((token != NULL) && (i < cnt))
  {
    i++;
    token = strtok(NULL, delim);
  }
  if (token != NULL)
  {
    strcpy(cpbuf, token);
    return(TRUE);
  }
  else
  {
    return(FALSE);
  }
}

/* mak_isalnum: Returns FALSE if not alnum or special, returns 1 if
     alnum, returns 2 if special.  Special characters are ".", "@",
     "-", or "/" */
int
mak_isalnum(ch)
  char ch;
{
  if (ch == '.' || ch == '@' || ch == '-' || ch == '/')
  {
    return (2);
  }

  return (isalnum(ch));
}

/* get_word: given a cp to a string, write a copy of the next "word"
    to the given buffer, skipping leading whitespace.  a "word" is
    some positive number of contiguous non-whitespace characters.
    next_cp is set to point to after the word, and the function
    returns the word if found, else NULL (with the dest buffer
    unchanged). */
char *
get_word(cp, buf)
  char *cp;
  char *buf;
{
  char *original_buf = buf;

  /* skip leading whitespace */
  while (*cp && isspace(*cp)) cp++;

  /* copy to buf upto next whitespace */
  while (*cp && !isspace(*cp))
  {
    *buf++ = *cp++;
  }

  /* if we accomplished something, terminate the 'buf' string, and
     advance 'cp' (the original text) beyond any trailing whitespace */
  if (buf != original_buf)
  {
    *buf = '\0';
    while (isspace(*cp)) cp++;
    return cp;
  }
  else
  {
    return NULL;
  }
}


/* paste: pastes the new info into the query string */
void
paste(line, cut_start, cut_end, rpl)
  char *line;
  char *cut_start;
  char *cut_end;
  char *rpl;

{
  char  *cp             = line;
  int   i               = 0;
  char  buf[MAX_LINE];

  while (cp <= cut_start)
  {
    buf[i++] = *cp;
    cp++;
  }
  buf[i] = '\0';
  strcat(buf, rpl);
  strcat(buf, cut_end);
  strcpy(line, buf);
}

/* strSTR: does a case-insensitve sub-string search */
char *
strSTR(str1, str2)       /*  by Jeff Odum  09/91  */
  char *str1;
  char *str2;
{
  int i;
  int j;
  int x;

  /* return immediately if NULL data */
  if (!str1 || !str2 || !*str1 || !*str2)
  {
    return NULL;
  }

  for (x = 0; str1[x]; x++)
  {
    if (toupper(str1[x]) == toupper(str2[0]))
    {
      /*  Matched first letter */
      for (i=x, j=0;
           (str1[i] && str2[j]) &&
           (toupper(str1[i]) == toupper(str2[j]));
           i++, j++ );

      if (!str2[j])
      {
        /*  Matched whole string  */
        return (&str1[x]);
      }
    }
  }

  return NULL;
}


/* record_separator: Returns true if it is a separator line, FALSE if not. NOTE: this function is redundant -- see new_record() */
int record_separator(line)
  char *line;
{
  return(new_record(line));
}

/* strupr: Upcases a string. */
char *
strupr(a)
  char *a;
{
  char *b;

  for (b = a; *a; a++)
  {
    *a = toupper(*a);
  }
  return (b);
}

/* on_off: return the ascii equivalents to the boolean argument */
char *
on_off(bool)
  int bool;
{
  static char *on  = "ON";
  static char *off = "OFF";

  if (bool)
  {
    return (on);
  }
  else
  {
    return (off);
  }
}

/* true_false: returns TRUE if the string is set to an acceptable true
      value: "true", "yes", 'on".  FALSE otherwise. */
int
true_false(bool)
  char *bool;
{
  trim(bool);
  
  if (STR_EQ(bool, "TRUE") || STR_EQ(bool, "YES") || STR_EQ(bool, "ON"))
  {
    return (TRUE);
  }
  else
  {
    return (FALSE);
  }
}

#include <errno.h>
#include <signal.h>

#define MAX_SET 20
/* run_program: runs the program 'program', using parameters 'param'.
   It also passes on the environment.  Currently, it doesn't return
   the status of the child process.  NOTE: I'm not sure why this isn't
   just a wrapper around system() yet. */
int
run_program(program, param)
  char *program;
  char *param;
{
  int           errno = 0;
  extern char   **environ;
  char          *myenviron[MAX_SET];
  char          *argv[MAX_SET];
  char          buf[MAX_LINE];
  int           i, pid;

  signal(SIGINT,  SIG_IGN);
  signal(SIGQUIT, SIG_IGN);

  /* enviroment stuff */
  for (i = 0; environ[i] != NULL && i < MAX_SET; i++)
  {
    myenviron[i] = NEW_STRING(environ[i]);
    strcpy(myenviron[i], environ[i]);
   }
  myenviron[i] = NULL;

  argv[0] = NEW_STRING(program);

  i = 0;
  while (delimstr(param, " \t", buf, i++) && i < MAX_SET)
  {
    argv[i] = NEW_STRING (buf);
  }
  argv[i] = NULL;

  pid = fork();

  if ( pid == -1 )
  {
    /* Fork failed. */
    if (errno == ENOMEM)
    {
      log_error("fork failed: memory allocation problem");
    }
    else
    {
      log_error("fork failed: %s", l_strerror(errno));
    }
  }
  else if (pid == 0)
  {
    /* Child. */
    execve(argv[0], argv, myenviron);
    log_error("exec failed: %s", l_strerror(errno));
    exit(-1);
  }
  else
  {
    /* Parent. */

    /* since we do not use the result code, and we have probably
       inherited the SIGCHLD, SIG_IGN state, skip the wait stuff. */

    /* int wait_ret; */
    /* while ( (wait_ret = wait(0)) != pid  && wait_ret != -1); */

    /* free the environment space */
    i = 0;
    while ( myenviron[i] != NULL)
    {
      free(myenviron[i++]);
    }

    i = 0;

    while ( argv[i] != NULL)
    {
      free(argv[i++]);
    }
  }
  return (TRUE);
}

/* get_tuple: parses a <tag1>:<tag2>:<datum> line. Returns TRUE if it
      succeeded, FALSE if not */
int
get_tuple(tag1, tag2, data, line)
  char *tag1;
  char *tag2;
  char *data;
  char *line;
{
  char str[MAX_LINE];

  if (line && *line && parse_line(line, tag1, str))
  {
    if (*str && parse_line(str, tag2 , data))
    {
      return TRUE;
    }
  }
  return FALSE;
}


