/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/

#ifndef _TYPES_H_
#define _TYPES_H_

/* includes */
#include <sys/types.h>
#include "llist.h"

typedef enum {
  FORMAT_FIELD_DEF,                   /* field definition */
  FORMAT_TEXT,                        /* text definition */
  FORMAT_CONDIT_TEXT,                 /* conditional text */
  FORMAT_AUTH_ELEM,                   /* print out the authority elem (soa) */
  FORMAT_DATA,                        /* data definition (placement) */
  FORMAT_SUB_FORMAT,                  /* definition of display for see-also */
  FORMAT_SUB_DEFER,                   /* defer for see-also */
  FORMAT_SUB_DEREF                    /* defererence a see-also */
} output_format_type;

typedef struct _output_format_struct {
  output_format_type type;               /* type of format */
  char *value;                           /* value of element (record) */
  char *multi_delim;                     /* delimiter for multi-valued fields */
  int  fixed_len;                        /* fixed length for element */
  int  sub_deref;                        /* dereferenece a see-also */
  int  sub_defer;                        /* defer referencing a see-also */
  char *sub_format;                      /* format of see-also */
  char *condit_pos;                      /* requires attribute */
  char *condit_neg;                      /* only if attribute not there */
  char *format_convert;                  /* format conversion utility */
  struct _output_format_struct *subitem; /* subitems for positioning */
  struct _output_format_struct *nxt;     /* next element on list */
  } output_format_struct;

#define NEW_OUTPUT_FORMAT (output_format_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(output_format_struct))

typedef struct _output_command_struct {
  char *command;
  int command_len;
  char *magic;
  char *description;
  char *header;
  char *footer;
  char *record_delim;
  struct _output_command_struct *nxt;
} output_command_struct;

#define NEW_OUTPUT_COMMAND (output_command_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(output_command_struct))

typedef struct _output_display_struct {
  char *command;
  int command_len;
  char *filename;
  output_format_struct *format;
  struct _output_display_struct *nxt;
} output_display_struct;

#define NEW_OUTPUT_DISPLAY (output_display_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(output_display_struct))

  typedef struct _define_struct {
  int   type;                      /* type of definition */
  char *name;                      /* name of the definition */
  char *value;                     /* value of the definition */
  struct _define_struct *nxt;      /* next definition */
} define_struct;

#define NEW_DEFINE (define_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(define_struct))

/* define the attributes within an object */
typedef enum
{
  INDEX_ALL_WORDS,
  INDEX_FIRST_WORD,
  INDEX_WORD_ONLY,
  INDEX_NO_WORDS
} a_index_type;

typedef enum
{
  TYPE_TEXT,
  TYPE_FILE,
  TYPE_MIME,
  TYPE_SEE_ALSO,
  TYPE_REFERRAL,
} a_type;

typedef struct _attribute_struct {
  int   global_id;                 /* unique id for all objects */
  int   internal_id;               /* unique id within the object */
  int   primary_key;               /* TRUE if primary key attribute */
  int   authority_area;            /* TRUE if a authority area attribute */
  int   soa;                       /* TRUE if a soa serial# attribute */
  int   required;                  /* TRUE if a required attribute */
  int   mult_lines;                /* TRUE if a can have mult lines */
  int   def_object;                /* TRUE if defines object */
  int   multiple_grp;              /* TRUE if defines multiple grouping */
  char *name;                      /* name of the attribute */
  char *name_alias;                /* alias name for the attribute */
  char *description;               /* description of the attribute */
  char *format;                    /* format of the object */
  char *object_name;               /* name of the object */
  a_index_type index_words;        /* what to index for rwaisindex */
  a_type type;                     /* type of attribute */
  int no_object_references;        /* number of linked list references */
  llist_struct *object_references; /* linked list of object references */
  struct _attribute_struct *nxt;   /* next item in list for that object */
} attribute_struct;

#define NEW_ATTRIBUTE (attribute_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(attribute_struct))

typedef enum {
    OBJECT,                  /* an object */
    REFERRAL,                /* an referral object */
    SOA,                     /* an soa record */
    EVERY_OBJECT,            /* every object (referral/soa/data) */
    NUM_OBJECTS              /* total number of objects (don't delete) */
} object_type;

/* object structure */
typedef struct _object_struct{
  char   *name;                             /* name of object */
  int    num;                               /* number of object */
  char   *attribute_fname;                  /* attribute template fname */
  char   *local_db;                         /* local wais db fname */
  char   *cache_db;                         /* cache wais db fname */
  char   *dbdir;                            /* cached object directory */
  char   *description;                      /* description of the database */
  int    type;                              /* type of object */
  attribute_struct *attributes;             /* ptr to list of attributes */
  output_display_struct *output_format;     /* ptr to list of output formats */
  void *commands;                           /* linked list of valid commands*/
  struct _object_struct *nxt;
  time_t    version;                        /* version of schema */
} object_struct;

#define NEW_OBJECT (object_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(object_struct))

/* command list */
typedef struct _old_command_struct {
  char *command;
  int length;
  object_struct *object;
  struct _old_command_struct *nxt;
} old_command_struct;

#define NEW_OLD_COMMAND (old_command_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(old_command_struct))
  
typedef enum {
  COMMAND_DISPLAY,                 /* command is a display command */
  COMMAND_OBJECT,                  /* command is a object command */
  COMMAND_QUERY,                   /* command is a query command */
  COMMAND_DIRECTIVE,               /* command is a server directive */
  COMMAND_X_DIRECTIVE,             /* command is a extended server directive */
  COMMAND_SPECIAL,                 /* command is a "special" command (help) */
  COMMAND_DISABLED_DIR,            /* command is a disabled directive */
  COMMAND_NUM                      /* don't delete */
} command_type;

typedef struct _command_struct  {
   char *command;                  /* command to initiate this action */
   int   len;                      /* min length to activate the command */
   char *magic;                    /* one char magic command */
   char *description;              /* description of command */
   int   arg_required;             /* argument required */
   command_type type;              /* type of command */
   object_struct  *object;         /* pointer to object */
   char *program;                  /* program to execute */
   int (*function)();              /* run function */
   char *syntax;                   /* syntax of the command */
   struct _command_struct *nxt;
} command_struct;

#define NEW_COMMAND (command_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(command_struct))

typedef struct _field_struct {
  int id;                      /* global_id of attribute */
  char *data;
  struct _field_struct *nxt;
} field_struct;

#define NEW_FIELD (field_struct *) calloc ((unsigned)1, \
  (unsigned)sizeof(field_struct))

typedef struct _record_struct {
   field_struct *fields;
   struct _record_struct *nxt;
} record_struct;

#define NEW_RECORD (record_struct *) calloc ((unsigned)1, \
  (unsigned)sizeof(record_struct))

typedef enum {
    LOGICAL_NULL,           /* null value - just in case */
    LOGICAL_AND,            /* an and boolean separator */
    LOGICAL_OR,             /* an or boolean separator */
    LOGICAL_NUM             /* total number of logicals (don't delete) */
} logical_separator_type;

/* query structure */
typedef struct _query_item_struct{
  char   *value;                    /* value to query for */
  char   *attribute;                /* attribute associated w/value */
  int    attribute_no;              /* attribute number assoc w/index */
  char   *display;                  /* display mode desired */
  object_struct  *object;           /* pointer to object desired */
  char   *directive;                /* server directive */
  int    (*function)();             /* function to be performed */
  int    loop_detect;               /* pass loop detection */
  char   *program;                  /* extended directive program */
  logical_separator_type separator; /* type of boolean separator */
  struct _query_item_struct *nxt;
} query_item_struct;

#define NEW_QUERY_ITEM (query_item_struct *) calloc ((unsigned)1, \
   (unsigned)sizeof(query_item_struct))
 
#endif /* _TYPES_H_ */



