/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include "delete.h"

#include <stdio.h>
#include <ctype.h>
#include <errno.h>

#include "client_msgs.h"
#include "defines.h"
#include "fileinfo.h"
#include "get_record.h"
#include "hitinfo.h"
#include "llist.h"
#include "log.h"
#include "mkdb.h"
#include "search.h"

/* local prototypes */

static int mkdb_mark_del_base_entry PROTO((int  target_file_no,
                                           long target_offset));

static void mark_as_del_buf PROTO((char *buf));

/**************************************************************************
  Function: This set of procedures does a linear search through the
  indexes, marks for delete any matches to file_no and offset
  returns TRUE on success 
          FALSE on failure
  By: Mark Kosters
**************************************************************************/
int
mkdb_del_index_entry (target_file_no, target_offset)
  int   target_file_no;
  long  target_offset;
{
  mkdb_fileinfo_struct  *fileinfo;
  char                  buf[MAX_BUF];
  char                  ndxvalue[MAX_BUF];
  long                  file_offset;
  int                   file_no;
  int                   deleted;
  FILE                  *fp;
  int                   index_no;
  long                  index_offset;
  int                   found = FALSE;

  index_no = 0;
  while ((fileinfo = get_fileinfo (ENUM_INDEX_FILE, index_no)) != NULL &&
         !found)
  {
    index_no = fileinfo->index_no+1;

    if ((fp = fopen (fileinfo->filename, "r+")) == NULL)
    {
      print_error (MISC_DIAG,
                   "mkdb_del_index_entry: can not open index file |%s|\n",
                   fileinfo->filename);
      return (FALSE);
    }

    /** necessary - starts infinite loop otherwise **/
    setbuf (fp, NULL);

    /** ok start scanning for data **/
    index_offset = 0 ;

    while (fgets(buf, MAX_BUF, fp) != NULL)
    {
      mkdb_break_out(buf, ndxvalue, &file_offset, &file_no, &deleted);
      if (target_file_no == file_no && file_offset == target_offset &&
          !deleted)
      {
        fseek (fp, index_offset, 0);

        /** bonzi! we found it **/
        found = TRUE;
#if DEBUG
        printf("mkdb_del_index_entry: printing %d:%d:%d:%s\n", file_offset, 
               file_no, TRUE, ndxvalue);
#endif
        fprintf(fp, "%ld:%d:%d:%s\n", file_offset, file_no, TRUE, ndxvalue);
      }

      index_offset = ftell(fp);
    }

    fclose (fp);
  }

  return (mkdb_mark_del_base_entry(target_file_no, target_offset));
}

/**************************************************************************
  Function: This procedure marks the record for delete by subsituting
  a  "_" has the first character of the object name.
  returns TRUE on success
          FALSE on failure
  By: Mark Kosters
**************************************************************************/
static int
mkdb_mark_del_base_entry(target_file_no, target_offset)
  int   target_file_no;
  long  target_offset;

{
  char                  *buf = NULL;
  int                   buf_size = 0;
  mkdb_fileinfo_struct  *cur;
  FILE                  *fp;

  buf = MKget_record(target_offset, target_file_no, buf, &buf_size, 0);
  if (buf && *buf)
  {
    mark_as_del_buf(buf);

    if ((cur = get_fileinfo (ENUM_DATA_FILE, target_file_no)) == NULL  ||
        (fp = fopen (cur->filename, "r+")) == NULL )
    {
      print_error(MISC_DIAG, "Unable to open %s: %s\n", cur->filename,
                  l_strerror(errno));
      return (FALSE);
    }

    fseek (fp, target_offset, 0);
    fprintf (fp, "%s", buf);
    fclose (fp);
  }

  return (TRUE);
}

/**************************************************************************
  Function: This procedure marks the buffer for delete by subsituting
  a  "_" has the first character of the object name.
  returns TRUE on success
          FALSE on failure
  By: Mark Kosters
**************************************************************************/
static void
mark_as_del_buf(buf)
  char *buf;
{
  *buf = '_';

  while (*buf)
  {
    if (*buf == '\n' && isalnum(*(buf+1)))
    {
      *(buf+1) = '_';
    }

    buf++;
  }
}

