/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include "mkdb_interface.h"

#include <stdio.h>
#include <stdlib.h>

#include "client_msgs.h"
#include "defines.h"
#include "fileinfo.h"
#include "get_record.h"
#include "hitinfo.h"
#include "mkdb.h"
#include "records.h"

#define MAX_BUF 4096

/*************************************************************************
  main routine
**************************************************************************/
record_struct *
mkdb_interface(database, query, maxhits, max_size, 
               error_code, records)
  char          *database;
  char          *query;
  int           maxhits;
  int           max_size;
  int           *error_code;
  record_struct *records;

{
  char                  fname[MAX_LINE];
  mkdb_hitinfo_struct   *hits;
  mkdb_hitinfo_struct   *cur;
  char                  *buf = NULL;
  int                   buf_size = 0;

  sprintf(fname, "%s.db", database);

  if (read_fileinfo(fname) == FALSE)
  {
    /* we couldn't find the *.db file, so we must assume that there
       just isn't any data here, and return normally */
    return(records);
  }
  
  hits = cur = mkdb_run_query(query, maxhits, error_code);
  
  mkdb_close_index();

  while (cur != NULL)
  {
    buf = MKget_record(cur->offset, cur->file_no, buf, &buf_size, max_size);
    records = read_record_from_text (records, buf, NULL);
    cur = cur->nxt;
  }

  free (buf);

  clear_hits (hits);
  if (*error_code)
  {
    print_error (EXCEEDED_MAXREC, "");
  }

  
  clear_fileinfo();

  return(records);
}

