/* insert std header here */

#include "daemon.h"

#include <stdio.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>
#include <errno.h>
#include <netdb.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#include "log.h"
#include "session.h"
#include "variables.h"

/* local globals */
static int sockfd;

int
run_daemon()
{
  struct sockaddr_in    client_addr;
  struct sockaddr_in    server_addr;
  int                   newsockfd;
  int                   clilen;
  int                   childpid;
  int                   one       = 1;
  int                   port      = get_port();

  
  if ((sockfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
  {
    log_error("run_daemon: Can not open socket: %s",
              l_strerror(errno));
    exit (1);
  }

  if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one))
      < 0)
  {
    log_error("run_daemon: Can not set socket options SO_REUSEADDR");
    exit(1);
  }
  
#ifdef NEED_LINGER  
  {
    struct linger li;
    
    li.l_onoff = 1;
    li.l_linger = 15;
    setsockopt(sockfd, SOL_SOCKET, SO_LINGER, (char *)&li,
               sizeof(struct linger));
  }
#endif  /* NEED_LINGER */

  /* for now, we will bind to all IP interfaces (INADDR_ANY) */
  /* Bind our local address so that the client can send to us */
  bzero((char *)&server_addr, sizeof(server_addr));
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons(port);
  server_addr.sin_addr.s_addr = htonl(INADDR_ANY);

  if (bind(sockfd, (struct sockaddr *)&server_addr, sizeof(server_addr)) < 0)
  {
    log_error("run_daemon: Can not bind socket: %s", l_strerror(errno));
    exit(1);
  }

  listen(sockfd, 5);

  /* Ignoring the SIGCHLD signal *should* solve the zombie process problem */
  signal(SIGCHLD,SIG_IGN);

  /* main loop: accepts a client connection and then forks off a child
     to handle it */
  for (;;)
  {
    clilen = sizeof(client_addr);
    newsockfd = accept(sockfd, (struct sockaddr *) &client_addr, &clilen);
    if (newsockfd < 0)
    {
      log_error("run_daemon: accept error: %s", l_strerror(errno));
      exit(1);
    }

    if ((childpid = fork()) <  0)
    {
      log_error("run_daemon: fork error: %s", l_strerror(errno));
      exit(1);
    }
    else if (childpid  ==  0)
    {
      /* this is the child */

      /* reset stdin and stdout to newsockfd */
      if (dup2(newsockfd, 0) == -1)
      {
        log_error("run_daemon: dup error: %s", l_strerror(errno));
        exit(1);
      }
      if (dup2(newsockfd, 1) == -1)
      {
        log_error("run_daemon: dup error: %s", l_strerror(errno));
        exit(1);
      }

      close(sockfd);
      run_session();
      close(newsockfd);

      exit(0);
    }
    else
    {
      /* else this is the parent */
      close(newsockfd);
    }
  } /* for (;;) */
}

void
close_main_socket()
{
  close(sockfd);
}
