/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "directive.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#include "defines.h"
#include "misc.h"
#include "client_msgs.h"
#include "parse.h"
#include "commands.h"
#include "security.h"
#include "log.h"

/** these are included for function names **/
#include "cache.h"
#include "client_info.h"
#include "commands.h"
#include "debug.h"
#include "define.h"
#include "display.h"
#include "exit.h"
#include "forward.h"
#include "help.h"
#include "holdconnect.h"
#include "limit.h"
#include "load.h"
#include "notify.h"
#include "object.h"
#include "private.h"
#include "register.h"
#include "schema.h"
#include "soa.h"
#include "status.h"
#include "xfer.h"

/* initializes the native commands */
int
initialize_directive_list()
{
  append_command("directive", 9, NULL, "directive", FALSE, COMMAND_DIRECTIVE, 
                 NULL, display_directive, NULL);
  append_command("object", 6, NULL, "object", FALSE, COMMAND_DIRECTIVE, NULL,
                 display_object, NULL);
  append_command("X-debug", 7, NULL, "debug mode", FALSE, COMMAND_DIRECTIVE,
                 NULL, set_debug, NULL);

#ifdef NO_REGISTER
  append_command("register", 8, NULL, "not implemented", FALSE,
                 COMMAND_DISABLED_DIR, NULL, directive_not_implemented, NULL);
#else
  append_command("register", 8, NULL, "register object", FALSE, 
                 COMMAND_DIRECTIVE, NULL, record_register, NULL);
#endif
  
  append_command("notify", 6, NULL, "notify", FALSE, COMMAND_DIRECTIVE, NULL,
                 record_notify, NULL);
  append_command("status", 6, NULL, "status", FALSE, COMMAND_DIRECTIVE, NULL,
                 show_status, NULL);
  append_command("cache", 5, NULL, "cache boolean", FALSE, COMMAND_DIRECTIVE, 
                 NULL, set_cache, NULL);
  append_command("soa", 3, NULL, "displays authoritative area for server", 
                 FALSE, COMMAND_DIRECTIVE, NULL, show_soa, NULL);
  append_command("load", 4, NULL, "displays system load average", FALSE, 
                 COMMAND_DIRECTIVE, NULL, display_system_load, NULL);
  append_command("schema", 6, NULL, "displays schema objects and attributes", 
                 FALSE, COMMAND_DIRECTIVE, NULL, display_schema, NULL);
  append_command("limit", 5, NULL, "displays and sets record hit limit", 
                 FALSE, COMMAND_DIRECTIVE, NULL, set_max_hit_limit, NULL);
  append_command("rwhois", 6, NULL, "identifies rwhois client", FALSE, 
                 COMMAND_DIRECTIVE, NULL, set_client_info, NULL);
  append_command("holdconnect", 11, NULL, "hold the connection boolean",
                 FALSE, COMMAND_DIRECTIVE, NULL, set_holdconnect, NULL);
  append_command("display", 7, NULL, "sets/displays the display types", FALSE, 
                 COMMAND_DIRECTIVE, NULL, set_or_display_display, NULL);
  append_command("xfer", 4, NULL, "transfer of objects/authority areas", 
                 FALSE, COMMAND_DIRECTIVE, NULL, transfer_data, NULL);
  append_command("quit", 4, NULL, "exit rwhois", FALSE, COMMAND_DIRECTIVE,
                 NULL, exit_whois, NULL);


  /* define directive really doesn't accomplish much yet, so disable it */

/*   append_command("define", 6, NULL, "define", FALSE, COMMAND_DIRECTIVE, */
/*               NULL, display_define, NULL); */
  append_command("define", 6, NULL, "not implemented", FALSE,
                 COMMAND_DISABLED_DIR, NULL, directive_not_implemented, NULL);

  /* language directive does nothing */
  append_command("language", 8, NULL, "language", FALSE,
                 COMMAND_DISABLED_DIR, NULL, directive_not_implemented, NULL);

  /* private does nothing useful (yet) */
/*   append_command("private", 7, NULL, "private", FALSE, COMMAND_DIRECTIVE, */
/*                  NULL, set_private, NULL); */
  append_command("private", 7, NULL, "not implemented", FALSE,
                 COMMAND_DISABLED_DIR, NULL, directive_not_implemented, NULL);

  /* forward doesn't work right yet */
/*   append_command("forward", 7, NULL, "forward", FALSE, COMMAND_DIRECTIVE, */
/*                  NULL, set_forward, NULL); */
  append_command("forward", 7, NULL, "not implemented", FALSE,
                 COMMAND_DISABLED_DIR, NULL, directive_not_implemented, NULL);
  
  return(TRUE);
}


/*************************************************************************
  returns TRUE if is directive command
          FALSE otherwise
**************************************************************************/
int
is_directive_from_str (str)
  char *str;
{
  if (str[0] == '-') {
    return (TRUE);
  }
  
  return (FALSE);
}

/*************************************************************************
  returns TRUE if is directive command
          FALSE otherwise
**************************************************************************/
int
is_directive (query_item)
  query_item_struct *query_item;
{
  if (query_item->directive != NULL) {
    return (TRUE);
  }

  return (FALSE);
}

/*************************************************************************
  creates the directive into a query structure
**************************************************************************/
query_item_struct *
create_directive(str)
  char *str;
{
  query_item_struct *new;
  char              word[MAX_LINE], *cp, *next;

  cp = str + 1; /* skips the "-" symbol */

  next = get_word (cp, word);

  new = NEW_QUERY_ITEM;
  new->directive = NEW_STRING (word);
  new->value = NEW_STRING (next);

  do_commands(word, next, new, TRUE);

  return (new);
}

/*************************************************************************
  runs the directive
  returns TRUE on success
          FALSE on failure
**************************************************************************/
int
run_directive (query_item)
  query_item_struct *query_item;
{
  if (query_item->function == NULL && query_item->program == NULL)
  {
    print_error (INVALID_DIRECTIVE, "");
    return FALSE;
  }
#if AUTHORIZATION_ON
  if (!authorized_directive(query_item->directive))
  {
    if (query_item->function)
    {
      print_error(UNAUTH_DIRECTIVE, "");
    }
    else
    {
      print_error(UNAUTH_X_DIRECTIVE, "");
    }

    log_entry (RWHOIS_LOG_FILE, "Ans: Directive DENIED");
    return FALSE;
  }
#endif
  if (!is_client_capable_of_action (query_item->directive))
  {
    print_error (UNCOMPAT_VERSION, "");
    return FALSE;
  }

  if (query_item->function)
  {
    return( (query_item->function)(query_item->value) );
  }

  if (run_program(query_item->program, query_item->value))
  {
    print_ok();
  }

  return TRUE;
}

/* displays the commands that are directives */
int
display_directive(str)
  char *str;
{
  command_struct *cur;
  int            cnt = 0;

  cur = return_all_commands();
  
  while (cur != NULL)
  {
    if (cur->type == COMMAND_DIRECTIVE || cur->type == COMMAND_X_DIRECTIVE)
    {
      if (cnt++)
      {
        printf ("%%directive\n");
      }
      printf ("%%directive directive: %s\n", cur->command);
      printf ("%%directive description: %s\n", cur->description);
      /** scott must not like these - again this is out of the spec *sigh* */
      /* printf ("%%directive length: %d\n", cur->len); */

      if (cur->syntax != NULL)
      {
        printf("%%directive syntax: %s\n", cur->syntax);
      }
    }
    cur = cur->nxt;
  }

  print_ok();

  return TRUE;
}

int
directive_not_implemented(str)
  char *str;
{
  print_error(UNIMPLEM_DIRECTIVE, "");

  return TRUE;
}
