/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "parse.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#include "attributes.h"
#include "commands.h"
#include "defines.h"
#include "directive.h"
#include "misc.h"
#include "objects.h"

/****************************************************************************
 local prototypes
****************************************************************************/

static query_item_struct *separate_predicates PROTO((char *str));
static void parse_out_attributes PROTO((query_item_struct *item));
static void cut_out PROTO((char *eq_ptr, char *att, char *val, char **start,
                           char **end, char *start_string));
static int attach_index PROTO((char *attrib, int *attrib_no, char *value,
                               char *buf));
static void debug_query_items PROTO((query_item_struct *cur));

/****************************************************************************
 parses the command into query items structure
 passes back a linked list of query_items
****************************************************************************/
query_item_struct *
encode_command(query_string)
  char *query_string;
{
  query_item_struct *query_items;
  query_item_struct *cur;

  /**
    first check to see if it is an directive
    **/

  if (is_directive_from_str(query_string))
  {
    query_items = create_directive(query_string);
#if DEBUG
    debug_query_items(query_items);
#endif
    return(query_items);
  }

  /**
    puts the query strings into predicates separated by booleans
    then parse out the attributes
    then check for commands
    **/

  cur = query_items = separate_predicates(query_string);
  while  (cur != NULL)
  {
    parse_out_attributes(cur);
    parse_out_commands(cur);
    cur=cur->nxt;
  }
#if DEBUG
  debug_query_items(query_items);
#endif
  return(query_items);
}

#define OR_SEPARATOR " or "
#define AND_SEPARATOR " and "
/****************************************************************************
 separates the query string into predicates based on logical operators
 passes back a linked list of query_items
****************************************************************************/
static query_item_struct *
separate_predicates(str)
  char *str;
{
  char                      *or_stmt;
  char                      *and_stmt;
  char                      *s;
  logical_separator_type    separator;
  query_item_struct         *top = NULL;

  do {
    separator = LOGICAL_NULL;
    or_stmt   = strSTR(str, OR_SEPARATOR);
    and_stmt  = strSTR(str, AND_SEPARATOR);
    
    if (and_stmt != NULL && or_stmt != NULL)
    {
      /* see which one is smaller */
      if (str-and_stmt > str-or_stmt)
        separator = LOGICAL_AND;
      else
        separator = LOGICAL_OR;
    }
    else if (and_stmt)
    {
      separator = LOGICAL_AND;
    }
    else if (or_stmt)
    {
      separator = LOGICAL_OR;
    }
    if (separator == LOGICAL_AND)
    {
      s = str;
      str = strSTR(str, AND_SEPARATOR);
      *str = '\0';
      str += strlen(AND_SEPARATOR);
      top = append_query(top, s, separator);
    }
    else if (separator == LOGICAL_OR)
    {
      s = str;
      str = strSTR (str, OR_SEPARATOR);
      *str = '\0';
      str+=strlen(OR_SEPARATOR);
      top = append_query(top, s, separator);
    }
  } while (separator != LOGICAL_NULL);

  top = append_query(top, str, LOGICAL_NULL);

  return (top);
}

/****************************************************************************
 appends the query to the list
****************************************************************************/
query_item_struct *
append_query (item, str, separator)
  query_item_struct      *item;
  char                   *str;
  logical_separator_type separator;

{
  query_item_struct *cur;
  query_item_struct *new;

  new            = NEW_QUERY_ITEM;
  new->value     = NEW_STRING(str);
  new->separator = separator;
  cur            = item;

  if (cur == NULL)
  {
    return(new);
  }
  while (cur->nxt != NULL)
  {
    cur=cur->nxt;
  }

  cur->nxt = new;

  return(item);
}

/****************************************************************************
 chages "[commands]attname = <data>" to "[commands]<data>" in the
 query_item value field
****************************************************************************/
static void
parse_out_attributes (item)
  query_item_struct *item;

{
  char  attrib[MAX_LINE];
  char  value[MAX_LINE];
  char  *cut_start;
  char  *cut_end;
  char  *cp;

  if ((cp = strchr( item->value, '=')) != NULL)
  {
    cut_out(cp, attrib, value, &cut_start, &cut_end, item->value);
    item->attribute = NEW_STRING(attrib);
    paste(item->value, cut_start, cut_end, value);
    /**
      if (!attach_index (attrib, value, buf, FALSE))
      return (FALSE);
      **/
  }
}

/****************************************************************************
 cuts out the attribute/value pairs returns ptrs in start and end
****************************************************************************/
static void
cut_out (eq_ptr, att, val, start, end, start_string)
  char  *eq_ptr;
  char  *att;
  char  *val;
  char  **start;
  char  **end;
  char  *start_string;
{
  char  *cp;
  int   i;

  *start = eq_ptr - 1;
  *end   = eq_ptr + 1;

  while (isspace(**start))
  {
    (*start)--;
  }
  while (mak_isalnum(**start) && *start >= start_string)
  {
    (*start)--;
  }
  while (isspace(**end))
  {
    (*end)++;
  }
  while (mak_isalnum (**end))
  {
    (*end)++;
  }

  cp = *start;
  i  = 0;
  while (cp < eq_ptr)
  {
    if (mak_isalnum (*cp))
    {
      att[i++] = *cp;
    }
    cp++;
  }

  att[i] = '\0';
  cp     = eq_ptr+1;
  i      = 0;

  while (cp < *end)
  {
    if (mak_isalnum(*cp))
    {
      val[i++] = *cp;
    }

    cp++;
  }

  val[i] ='\0';
}

/****************************************************************************
 decodes the query into a string wais likes
****************************************************************************/
char *
decode_command_list(str, query_list)
  char              *str;
  query_item_struct *query_list;
{
  char buf[MAX_LINE];

#if DEBUG
  debug_query_items(query_list);
#endif
  *str = '\0';
  while (query_list != NULL)
  {
    /* skip if no value */
    if (query_list->value != NULL)
    {
      if (query_list->attribute != NULL &&
          attach_index (query_list->attribute, &(query_list->attribute_no),
                        query_list->value, buf))
      {
        strcat(str, buf);
      }
      else
      {
        strcat(str, query_list->value);
      }

      switch (query_list->separator)
      {
      case LOGICAL_AND:
        strcat(str, AND_SEPARATOR);
        break;
      case LOGICAL_OR:
        strcat(str, OR_SEPARATOR);
        break;
      default:
        break;
      }
    }

    query_list = query_list->nxt;
  }

#if DEBUG
  printf ("decode_command_list: |%s|\n", str);
#endif

  return(str);
}

/****************************************************************************
 decodes the query into a string for authority element
****************************************************************************/
char *
decode_data_for_authority_element (query_list)
  query_item_struct *query_list;
{
  static char str[MAX_LINE];

  *str = '\0';

  while (query_list != NULL)
  {
    /* skip if no value */
    if (query_list->value != NULL)
    {
      strcat(str, query_list->value);
      switch (query_list->separator)
      {
      case LOGICAL_AND:
        strcat (str, AND_SEPARATOR);
        break;
      case LOGICAL_OR:
        strcat (str, OR_SEPARATOR);
        break;
      default:
        break;
      }
    }

    query_list = query_list->nxt;
  }

  return (str);
}

/****************************************************************************
 attaches the unique index id's to look for
 returns FALSE if not found or no attrib/val combos
****************************************************************************/
static int
attach_index(attrib, attrib_no, value, buf)
  char  *attrib;
  int   *attrib_no;
  char  *value;
  char  *buf;

{
  int   index_id;
  
  if ((index_id =  get_attribute_global_id_from_txt (attrib)) == -1)
  {
    return (FALSE);
  }

  *attrib_no = index_id;
  sprintf (buf,"%d%s%s", index_id, INDEX_PRFX, value);

  return (TRUE);
}

/****************************************************************************
 deletes the query list
****************************************************************************/
void
delete_query_list (item)
  query_item_struct *item;
{
  query_item_struct *del;

  while (item != NULL)
  {
    del = item;
    item=item->nxt;
    free (del->value);
    free (del->attribute);
    free (del->attribute);
    free (del->display);
    free (del->directive);
    free (del);
  }
}

/****************************************************************************
 parses the command list - returns first display item
 returns NULL if does not exist
****************************************************************************/
char *
get_display_mode (query_item)
  query_item_struct *query_item;
{
  while (query_item != NULL)
  {
    if (query_item->display != NULL)
    {
      return (query_item->display);
    }

    query_item = query_item->nxt;
  }
  
  return (NULL);
}

/****************************************************************************
  debugs query items
****************************************************************************/
static void
debug_query_items(cur)
  query_item_struct *cur;
{
  while (cur != NULL)
  {
    printf("debug_query_items: value |%s|\n", cur->value);
    printf("debug_query_items: attribute |%s|\n", cur->attribute);
    printf("debug_query_items: display |%s|\n", cur->display);
    printf("debug_query_items: directive |%s|\n", cur->directive);
    if (cur->object != NULL)
    {
      printf ("debug_query_items: object |%s|\n", cur->object->name);
    }
    else
    {
      printf ("debug_query_items: object |(null)|\n");
    }
    printf("debug_query_items: separator |%d|\n---\n", cur->separator);
    cur = cur->nxt;
  }
}
